#include "USBSimulationDataGenerator.h"
#include "USBAnalyzerSettings.h"

#include "USBTypes.h"

const double MILLISEC = 1 / 1e3;
const double LS_BIT_DUR_S = 1 / 15e5;
const double FS_BIT_DUR_S = 1 / 12e6;

const U16 SIM_END_OF_DATA	= 0xffff;
const U16 SIM_NEW_FRAME		= 0xfffe;
const U16 SIM_NEW_PACKET	= 0xfffd;
const U16 SIM_RESET			= 0xfffc;
const U16 SIM_LOOP_HERE		= 0xfffb;

const U16 SimDataLS[] = {
SIM_RESET,
SIM_NEW_FRAME,
SIM_NEW_FRAME,
SIM_NEW_FRAME,
SIM_NEW_FRAME,
0x80, 0x2D, 0x00, 0x10, SIM_NEW_PACKET,
0x80, 0xC3, 0x00, 0x05, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0xEB, 0x25, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x00, 0x10, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x00, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x2D, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x80, 0x06, 0x00, 0x01, 0x00, 0x00, 0x12, 0x00, 0xE0, 0xF4, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x12, 0x01, 0x10, 0x01, 0x00, 0x00, 0x00, 0x08, 0x11, 0x77, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x3C, 0x41, 0x03, 0x20, 0x06, 0x03, 0x01, 0x02, 0x1C, 0x9C, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x01, 0x3F, 0x8F, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0xE1, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x00, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x2D, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x80, 0x06, 0x00, 0x02, 0x00, 0x00, 0xFF, 0x00, 0xE9, 0xA4, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x09, 0x02, 0x22, 0x00, 0x01, 0x01, 0x00, 0xA0, 0x0A, 0x98, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x23, 0x09, 0x04, 0x00, 0x00, 0x01, 0x03, 0x01, 0xF5, 0x8D, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x01, 0x00, 0x09, 0x21, 0x10, 0x01, 0x00, 0x01, 0x56, 0x66, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x22, 0x41, 0x00, 0x07, 0x05, 0x81, 0x03, 0x08, 0x89, 0x23, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x18, 0xFE, 0x45, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0xE1, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x00, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x2D, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x80, 0x06, 0x00, 0x03, 0x00, 0x00, 0xFF, 0x00, 0xD4, 0x64, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x04, 0x03, 0x09, 0x04, 0x09, 0x78, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0xE1, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x00, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x2D, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x80, 0x06, 0x02, 0x03, 0x09, 0x04, 0xFF, 0x00, 0x97, 0xDB, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x24, 0x03, 0x44, 0x00, 0x65, 0x00, 0x6C, 0x00, 0xB3, 0x97, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x6C, 0x00, 0x20, 0x00, 0x55, 0x00, 0x53, 0x00, 0x93, 0xD5, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x42, 0x00, 0x20, 0x00, 0x4B, 0x00, 0x65, 0x00, 0x01, 0xC9, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x79, 0x00, 0x62, 0x00, 0x6F, 0x00, 0x61, 0x00, 0x44, 0x7C, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x72, 0x00, 0x64, 0x00, 0xCE, 0x63, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0xE1, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x00, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, 
0x80, 0x2D, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x80, 0x06, 0x00, 0x01, 0x00, 0x00, 0x12, 0x00, 0xE0, 0xF4, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x12, 0x01, 0x10, 0x01, 0x00, 0x00, 0x00, 0x08, 0x11, 0x77, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x3C, 0x41, 0x03, 0x20, 0x06, 0x03, 0x01, 0x02, 0x1C, 0x9C, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x01, 0x3F, 0x8F, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0xE1, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x00, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
SIM_NEW_FRAME,
0x80, 0x2D, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x80, 0x06, 0x00, 0x02, 0x00, 0x00, 0x09, 0x00, 0xAE, 0x04, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x09, 0x02, 0x22, 0x00, 0x01, 0x01, 0x00, 0xA0, 0x0A, 0x98, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x23, 0x01, 0x66, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0xE1, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x00, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
SIM_NEW_FRAME,
0x80, 0x2D, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x80, 0x06, 0x00, 0x02, 0x00, 0x00, 0x22, 0x00, 0xB0, 0xF4, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x09, 0x02, 0x22, 0x00, 0x01, 0x01, 0x00, 0xA0, 0x0A, 0x98, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x23, 0x09, 0x04, 0x00, 0x00, 0x01, 0x03, 0x01, 0xF5, 0x8D, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x01, 0x00, 0x09, 0x21, 0x10, 0x01, 0x00, 0x01, 0x56, 0x66, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x22, 0x41, 0x00, 0x07, 0x05, 0x81, 0x03, 0x08, 0x89, 0x23, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x18, 0xFE, 0x45, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0xE1, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x00, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x2D, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x00, 0x09, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x27, 0x25, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x00, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x2D, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x21, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xD6, 0x20, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x00, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x2D, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x81, 0x06, 0x00, 0x22, 0x00, 0x00, 0x81, 0x00, 0x88, 0x0F, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x05, 0x01, 0x09, 0x06, 0xA1, 0x01, 0x05, 0x07, 0xD7, 0x3C, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x19, 0xE0, 0x29, 0xE7, 0x15, 0x00, 0x25, 0x01, 0x73, 0xEF, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x75, 0x01, 0x95, 0x08, 0x81, 0x02, 0x95, 0x01, 0xB3, 0x87, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x75, 0x08, 0x81, 0x01, 0x95, 0x03, 0x75, 0x01, 0xE8, 0x62, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x05, 0x08, 0x19, 0x01, 0x29, 0x03, 0x91, 0x02, 0xDD, 0xCF, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x95, 0x01, 0x75, 0x05, 0x91, 0x01, 0x95, 0x06, 0x33, 0x6C, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x75, 0x08, 0x15, 0x00, 0x26, 0xFF, 0x00, 0x05, 0x08, 0x91, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x07, 0x19, 0x00, 0x2A, 0xFF, 0x00, 0x81, 0x00, 0x3F, 0x51, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0xC0, 0x40, 0xEF, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
SIM_NEW_FRAME,
0x80, 0xE1, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x00, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_LOOP_HERE,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x2D, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x21, 0x09, 0x00, 0x02, 0x00, 0x00, 0x01, 0x00, 0x9D, 0x70, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0xE1, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x01, 0x81, 0x7F, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x00, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_END_OF_DATA,
};

const U16 SimDataFS[] = {
SIM_RESET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
//SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
//SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x2D, 0x00, 0x10, SIM_NEW_PACKET,
0x80, 0xC3, 0x80, 0x06, 0x00, 0x01, 0x00, 0x00, 0x40, 0x00, 0xDD, 0x94, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x00, 0x10, SIM_NEW_PACKET,
0x80, 0x4B, 0x12, 0x01, 0x00, 0x02, 0x00, 0x00, 0x00, 0x08, 0x57, 0xE7, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
SIM_NEW_FRAME,
0x80, 0xE1, 0x00, 0x10, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x00, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
//SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
//SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x2D, 0x00, 0x10, SIM_NEW_PACKET,
0x80, 0xC3, 0x00, 0x05, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0xEB, 0x25, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x00, 0x10, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x00, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
//SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
//SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
//SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x2D, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x80, 0x06, 0x00, 0x01, 0x00, 0x00, 0x12, 0x00, 0xE0, 0xF4, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x12, 0x01, 0x00, 0x02, 0x00, 0x00, 0x00, 0x08, 0x57, 0xE7, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x32, 0x15, 0x16, 0x00, 0x00, 0x01, 0x01, 0x02, 0xAA, 0x9F, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x01, 0x3F, 0x8F, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0xE1, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x00, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x2D, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x80, 0x06, 0x00, 0x02, 0x00, 0x00, 0xFF, 0x00, 0xE9, 0xA4, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x09, 0x02, 0x22, 0x00, 0x01, 0x01, 0x00, 0xA0, 0x0A, 0x98, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x32, 0x09, 0x04, 0x00, 0x00, 0x01, 0x03, 0x01, 0x35, 0x4D, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x02, 0x00, 0x09, 0x21, 0x11, 0x01, 0x00, 0x01, 0x17, 0x8F, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x22, 0x49, 0x00, 0x07, 0x05, 0x81, 0x03, 0x08, 0x00, 0xE3, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x08, 0xFF, 0x89, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
SIM_NEW_FRAME,
0x80, 0xE1, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x00, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
SIM_NEW_FRAME,
0x80, 0x2D, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x80, 0x06, 0x00, 0x03, 0x00, 0x00, 0xFF, 0x00, 0xD4, 0x64, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x04, 0x03, 0x09, 0x04, 0x09, 0x78, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
SIM_NEW_FRAME,
0x80, 0xE1, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x00, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
SIM_NEW_FRAME,
0x80, 0x2D, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x80, 0x06, 0x02, 0x03, 0x09, 0x04, 0xFF, 0x00, 0x97, 0xDB, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x22, 0x03, 0x52, 0x00, 0x61, 0x00, 0x7A, 0x00, 0x3E, 0x1B, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x65, 0x00, 0x72, 0x00, 0x20, 0x00, 0x44, 0x00, 0x4B, 0xF1, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x65, 0x00, 0x61, 0x00, 0x74, 0x00, 0x68, 0x00, 0x45, 0xA2, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x41, 0x00, 0x64, 0x00, 0x64, 0x00, 0x65, 0x00, 0x46, 0x4C, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x72, 0x00, 0xDA, 0xEF, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
SIM_NEW_FRAME,
0x80, 0xE1, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x00, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
SIM_NEW_FRAME,
0x80, 0x2D, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x80, 0x06, 0x00, 0x06, 0x00, 0x00, 0x0A, 0x00, 0x5F, 0x34, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x1E, SIM_NEW_PACKET,
SIM_NEW_FRAME,
SIM_NEW_FRAME,
SIM_NEW_FRAME,
0x80, 0x2D, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x80, 0x06, 0x00, 0x01, 0x00, 0x00, 0x12, 0x00, 0xE0, 0xF4, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x12, 0x01, 0x00, 0x02, 0x00, 0x00, 0x00, 0x08, 0x57, 0xE7, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x32, 0x15, 0x16, 0x00, 0x00, 0x01, 0x01, 0x02, 0xAA, 0x9F, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x01, 0x3F, 0x8F, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0xE1, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x00, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x2D, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x80, 0x06, 0x00, 0x02, 0x00, 0x00, 0x09, 0x00, 0xAE, 0x04, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x09, 0x02, 0x22, 0x00, 0x01, 0x01, 0x00, 0xA0, 0x0A, 0x98, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x32, 0xC1, 0x6A, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0xE1, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x00, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x2D, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x80, 0x06, 0x00, 0x02, 0x00, 0x00, 0x22, 0x00, 0xB0, 0xF4, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x09, 0x02, 0x22, 0x00, 0x01, 0x01, 0x00, 0xA0, 0x0A, 0x98, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x32, 0x09, 0x04, 0x00, 0x00, 0x01, 0x03, 0x01, 0x35, 0x4D, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x02, 0x00, 0x09, 0x21, 0x11, 0x01, 0x00, 0x01, 0x17, 0x8F, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x22, 0x49, 0x00, 0x07, 0x05, 0x81, 0x03, 0x08, 0x00, 0xE3, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x08, 0xFF, 0x89, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0xE1, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x00, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
SIM_NEW_FRAME,
0x80, 0x2D, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x00, 0x09, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x27, 0x25, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x00, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x2D, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x21, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xD6, 0x20, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x00, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x2D, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x81, 0x06, 0x00, 0x22, 0x00, 0x00, 0x89, 0x00, 0x8F, 0xCF, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x05, 0x01, 0x09, 0x02, 0xA1, 0x01, 0x09, 0x01, 0xA3, 0xFE, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0xA1, 0x00, 0x05, 0x09, 0x19, 0x01, 0x29, 0x07, 0xA0, 0xDA, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x15, 0x00, 0x25, 0x01, 0x95, 0x08, 0x75, 0x01, 0x0F, 0x6C, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x81, 0x02, 0x06, 0x00, 0xFF, 0x09, 0x40, 0x95, 0x44, 0x47, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x02, 0x75, 0x08, 0x15, 0x81, 0x25, 0x7F, 0x81, 0xCF, 0xC6, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x02, 0x05, 0x01, 0x09, 0x38, 0x15, 0x81, 0x25, 0x0B, 0xD2, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x7F, 0x75, 0x08, 0x95, 0x01, 0x81, 0x06, 0x09, 0x43, 0x70, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0x30, 0x09, 0x31, 0x16, 0x00, 0x80, 0x26, 0xFF, 0x33, 0xCA, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x7F, 0x75, 0x10, 0x95, 0x02, 0x81, 0x06, 0xC0, 0x80, 0xBA, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
0x80, 0x69, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0xC3, 0xC0, 0x40, 0xEF, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME,
SIM_NEW_FRAME,
0x80, 0xE1, 0x01, 0xE8, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x00, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_LOOP_HERE,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0xC3, 0x00, 0x00, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xAB, 0xE0, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0xC3, 0x00, 0x00, 0xFD, 0x00, 0x00, 0x00, 0x03, 0x00, 0xAA, 0xE9, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x01, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0xAE, 0xC8, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0xC3, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0xFE, 0xFF, 0xBE, 0x36, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x03, 0x0A, 0x00, 0x03, 0x00, 0xF6, 0xFF, 0x8B, 0xFA, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0xC3, 0x00, 0x04, 0x10, 0x00, 0x04, 0x00, 0xF0, 0xFF, 0xFD, 0xD4, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0xFF, 0x1B, 0x00, 0xFF, 0xFF, 0xE5, 0xFF, 0xB9, 0xE4, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0xC3, 0x00, 0xFD, 0x23, 0x00, 0xFD, 0xFF, 0xDD, 0xFF, 0x8C, 0xE4, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0xFF, 0x1B, 0x00, 0xFF, 0xFF, 0xE5, 0xFF, 0xB9, 0xE4, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0xC3, 0x00, 0x00, 0x11, 0x00, 0x00, 0x00, 0xEF, 0xFF, 0xB0, 0xC5, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x00, 0x05, 0x00, 0x00, 0x00, 0xFB, 0xFF, 0xBC, 0xD1, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0xC3, 0x00, 0x00, 0xFE, 0x00, 0x00, 0x00, 0x02, 0x00, 0xAB, 0x4A, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x00, 0xFC, 0x00, 0x00, 0x00, 0x04, 0x00, 0xA9, 0x08, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0xC3, 0x00, 0x06, 0xF5, 0x00, 0x06, 0x00, 0x0B, 0x00, 0xCA, 0xE9, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x0A, 0xEC, 0x00, 0x0A, 0x00, 0x14, 0x00, 0x0F, 0x80, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0xC3, 0x00, 0x0C, 0xE9, 0x00, 0x0C, 0x00, 0x17, 0x00, 0x69, 0xAD, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x0C, 0xE5, 0x00, 0x0C, 0x00, 0x1B, 0x00, 0x6C, 0x61, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0xC3, 0x00, 0x0F, 0xE9, 0x00, 0x0F, 0x00, 0x17, 0x00, 0x5A, 0xE9, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x0D, 0xE5, 0x00, 0x0D, 0x00, 0x1B, 0x00, 0x7D, 0x5D, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0xC3, 0x00, 0x0D, 0xEC, 0x00, 0x0D, 0x00, 0x14, 0x00, 0x78, 0x34, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x0C, 0xED, 0x00, 0x0C, 0x00, 0x13, 0x00, 0x6A, 0xE9, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0xC3, 0x00, 0x0A, 0xF1, 0x00, 0x0A, 0x00, 0x0F, 0x00, 0x06, 0xFD, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x05, 0xF3, 0x00, 0x05, 0x00, 0x0D, 0x00, 0xFA, 0x6B, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0xC3, 0x00, 0x04, 0xF8, 0x00, 0x04, 0x00, 0x08, 0x00, 0xE9, 0x7C, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x00, 0xFC, 0x00, 0x00, 0x00, 0x04, 0x00, 0xA9, 0x08, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0xC3, 0x00, 0x00, 0xFE, 0x00, 0x00, 0x00, 0x02, 0x00, 0xAB, 0x4A, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0xFE, 0x02, 0x00, 0xFE, 0xFF, 0xFE, 0xFF, 0xA0, 0x21, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0xC3, 0x00, 0xFD, 0x00, 0x00, 0xFD, 0xFF, 0x00, 0x00, 0x92, 0xA7, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0xFA, 0x02, 0x00, 0xFA, 0xFF, 0xFE, 0xFF, 0xE4, 0xD1, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0xC3, 0x00, 0xFB, 0x05, 0x00, 0xFB, 0xFF, 0xFB, 0xFF, 0xF7, 0x0A, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0xFB, 0x04, 0x00, 0xFB, 0xFF, 0xFC, 0xFF, 0xF4, 0xEB, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0xC3, 0x00, 0xFF, 0x07, 0x00, 0xFF, 0xFF, 0xF9, 0xFF, 0xB3, 0x78, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x00, 0x04, 0x00, 0x00, 0x00, 0xFC, 0xFF, 0xBF, 0x30, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0xC3, 0x00, 0x03, 0x02, 0x00, 0x03, 0x00, 0xFE, 0xFF, 0x8D, 0x72, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x06, 0x04, 0x00, 0x06, 0x00, 0xFC, 0xFF, 0xD9, 0xB8, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0xC3, 0x00, 0x09, 0x03, 0x00, 0x09, 0x00, 0xFD, 0xFF, 0x25, 0x8B, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x0E, 0x03, 0x00, 0x0E, 0x00, 0xFD, 0xFF, 0x52, 0x3F, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0xC3, 0x00, 0x12, 0x01, 0x00, 0x12, 0x00, 0xFF, 0xFF, 0x88, 0xEC, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x14, 0x01, 0x00, 0x14, 0x00, 0xFF, 0xFF, 0xEE, 0x64, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0xC3, 0x00, 0x13, 0xFF, 0x00, 0x13, 0x00, 0x01, 0x00, 0x8C, 0x2E, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x14, 0xFF, 0x00, 0x14, 0x00, 0x01, 0x00, 0xFB, 0x9A, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0xC3, 0x00, 0x13, 0xFF, 0x00, 0x13, 0x00, 0x01, 0x00, 0x8C, 0x2E, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x0D, 0xFE, 0x00, 0x0D, 0x00, 0x02, 0x00, 0x75, 0x26, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0xC3, 0x00, 0x0B, 0xFD, 0x00, 0x0B, 0x00, 0x03, 0x00, 0x12, 0x0D, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x4B, 0x00, 0x04, 0xFE, 0x00, 0x04, 0x00, 0x02, 0x00, 0xEF, 0xBA, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0xC3, 0x00, 0x02, 0xFF, 0x00, 0x02, 0x00, 0x01, 0x00, 0x88, 0x13, SIM_NEW_PACKET,
0x80, 0xD2, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME, SIM_NEW_FRAME,
0x80, 0x69, 0x81, 0x58, SIM_NEW_PACKET,
0x80, 0x5A, SIM_NEW_PACKET,
SIM_END_OF_DATA,
};

USBSimulationDataGenerator::USBSimulationDataGenerator()
{}

USBSimulationDataGenerator::~USBSimulationDataGenerator()
{}

void USBSimulationDataGenerator::Initialize(U32 simulation_sample_rate, USBAnalyzerSettings* settings)
{
	mSimulationSampleRateHz = simulation_sample_rate;
	mSettings = settings;

	if (mSettings->mSpeed == LOW_SPEED)
		mpCurrSimData = SimDataLS;
	else
		mpCurrSimData = SimDataFS;

	mAccuDur = 0;
	mFrameCnt = 1;

	mClockGenerator.Init(simulation_sample_rate / 10, simulation_sample_rate);

	mDP = mUSBSimulationChannels.Add(settings->mDPChannel, mSimulationSampleRateHz, BIT_LOW);
	mDM = mUSBSimulationChannels.Add(settings->mDMChannel, mSimulationSampleRateHz, BIT_LOW);

	mUSBSimulationChannels.AdvanceAll(mClockGenerator.AdvanceByTimeS(.001));	// insert a pause before we start
}

U32 USBSimulationDataGenerator::GenerateSimulationData(U64 largest_sample_requested, U32 sample_rate, SimulationChannelDescriptor** simulation_channels)
{
	U64 adjusted_largest_sample_requested = AnalyzerHelpers::AdjustSimulationTargetSample(largest_sample_requested, sample_rate, mSimulationSampleRateHz);

	const double BIT_DUR_S = mSettings->mSpeed == LOW_SPEED ? LS_BIT_DUR_S : FS_BIT_DUR_S;

	while (mDP->GetCurrentSampleNumber() < adjusted_largest_sample_requested)
	{
		if (*mpCurrSimData == SIM_NEW_FRAME)
		{
			OutFillFrame();			// fill with J state up to 1ms

			if (mSettings->mSpeed == LOW_SPEED)
			{
				OutLSKeepAlive();			// keep-alive
				OutJ(LS_BIT_DUR_S * 4);		// and a 4 bit J state
			} else {
				OutFSSOF();
				OutJ(FS_BIT_DUR_S * 2);		// 2 bit J state
			}
	
			++mpCurrSimData;

		} else if (*mpCurrSimData == SIM_NEW_PACKET) {

			// add a 2 bit pause
			if (mSettings->mSpeed == LOW_SPEED)
				OutJ(LS_BIT_DUR_S * 2);
			else
				OutJ(FS_BIT_DUR_S * 2);

			++mpCurrSimData;

		} else if (*mpCurrSimData == SIM_RESET) {

			// output a reset
			OutReset();

			// a little bit of silence
			OutJ(MILLISEC / 20);

			// start a USB frame
			ResetFrameDuration();

			++mpCurrSimData;

		} else if (*mpCurrSimData == SIM_LOOP_HERE) {

			// remember the position in the stream, so that we can return here after
			// we ran out of data
			mpSimDataLoopHere = mpCurrSimData;
			++mpCurrSimData;

		} else if (*mpCurrSimData == SIM_END_OF_DATA) {

			// restart from the remembered position
			mpCurrSimData = mpSimDataLoopHere;

		} else {
			// output the packet
			mpCurrSimData = OutPacket(mpCurrSimData);
		}
	}

	*simulation_channels = mUSBSimulationChannels.GetArray();

	return mUSBSimulationChannels.GetCount();
}

void USBSimulationDataGenerator::OutReset()
{
	OutSE0(MILLISEC * 15);
}

void USBSimulationDataGenerator::SetJ()
{
	if (mSettings->mSpeed == LOW_SPEED)
	{
		mDP->TransitionIfNeeded(BIT_LOW);
		mDM->TransitionIfNeeded(BIT_HIGH);
	} else {
		mDP->TransitionIfNeeded(BIT_HIGH);
		mDM->TransitionIfNeeded(BIT_LOW);
	}
}

void USBSimulationDataGenerator::SetK()
{
	if (mSettings->mSpeed == LOW_SPEED)
	{
		mDP->TransitionIfNeeded(BIT_HIGH);
		mDM->TransitionIfNeeded(BIT_LOW);
	} else {
		mDP->TransitionIfNeeded(BIT_LOW);
		mDM->TransitionIfNeeded(BIT_HIGH);
	}
}

void USBSimulationDataGenerator::OutFillFrame()
{
	// fill the frame
	OutJ(MILLISEC - mAccuDur);

	// reset the frame duration counter
	ResetFrameDuration();
}

void USBSimulationDataGenerator::OutSE0(const double dur)
{
	// set the states
	mDP->TransitionIfNeeded(BIT_LOW);
	mDM->TransitionIfNeeded(BIT_LOW);

	// advance the time
	mUSBSimulationChannels.AdvanceAll(mClockGenerator.AdvanceByTimeS(dur));

	// update the frame timer
	mAccuDur += dur;
}

void USBSimulationDataGenerator::OutK(const double dur)
{
	SetK();

	// advance by time
	mUSBSimulationChannels.AdvanceAll(mClockGenerator.AdvanceByTimeS(dur));

	mAccuDur += dur;
}

void USBSimulationDataGenerator::OutJ(const double dur)
{
	SetJ();

	// advance by time
	mUSBSimulationChannels.AdvanceAll(mClockGenerator.AdvanceByTimeS(dur));

	mAccuDur += dur;
}

void USBSimulationDataGenerator::OutLSKeepAlive()
{
	OutSE0(LS_BIT_DUR_S * 2);
}

const U16* USBSimulationDataGenerator::OutPacket(const U16* pPacket)
{
	SetJ();		// set to idle state

	const double BIT_DUR = mSettings->mSpeed == LOW_SPEED ? LS_BIT_DUR_S : FS_BIT_DUR_S;

	int one_cnt = 0;	// number of consecutive ones
	int total_bits = 0;
	while (*pPacket <= 0xff)
	{
		U8 byte = U8(*pPacket);

		for (int bitc = 0; bitc < 8; ++bitc, byte >>= 1)
		{
			if (byte & 1)
			{
				// we have a one - no transition
				++one_cnt;

			} else {

				// we have a zero
				mDP->Transition();
				mDM->Transition();

				one_cnt = 0;
			}

			mUSBSimulationChannels.AdvanceAll(mClockGenerator.AdvanceByTimeS(BIT_DUR));

			++total_bits;

			// add a stuff bit?
			if (one_cnt == 6)
			{
				mDP->Transition();
				mDM->Transition();

				mUSBSimulationChannels.AdvanceAll(mClockGenerator.AdvanceByTimeS(BIT_DUR));

				one_cnt = 0;

				++total_bits;
			}
		}

		++pPacket;
	}

	// add the EOP
	OutSE0(BIT_DUR * 2);
	OutJ(BIT_DUR);

	// add the length of the packet (including EOP)
	mAccuDur += total_bits * BIT_DUR;

	// return the byte pointer
	return pPacket;
}

void USBSimulationDataGenerator::OutFSSOF()
{
	U16 packet[5];

	packet[0] = 0x80;		// SYNC
	packet[1] = PID_SOF;

	U16 pckt2_3 = (USBPacket::CalcCRC5(mFrameCnt) << 11) | mFrameCnt;

	++mFrameCnt;

	packet[2] = pckt2_3 & 0xff;
	packet[3] = pckt2_3 >> 8;
	packet[4] = SIM_NEW_PACKET;

	// output the packet
	OutPacket(packet);
}
